import { Context } from "@serverless-stack/node/context/index.js";
import { Logger } from "../logger/index.js";
export const useBus = Context.memo(() => {
    const subscriptions = {};
    function subscribers(type) {
        let arr = subscriptions[type];
        if (!arr) {
            arr = [];
            subscriptions[type] = arr;
        }
        return arr;
    }
    return {
        publish(type, properties) {
            const payload = {
                type,
                properties,
            };
            Logger.debug(`Publishing event: ${JSON.stringify(payload, null, 4)}`);
            for (const sub of subscribers(type))
                sub.cb(payload);
        },
        unsubscribe(sub) {
            const arr = subscribers(sub.type);
            const index = arr.indexOf(sub);
            if (!index)
                return;
            arr.splice(index, 1);
        },
        subscribe(type, cb) {
            const sub = {
                type,
                cb,
            };
            subscribers(type).push(sub);
            return sub;
        },
    };
});
